@@ -59,6 +59,7 @@ enum e1d_line_ctr {
 	LINE_CTR_RX_REMOTE_A,
 	LINE_CTR_FRAMES_MUXED_E1T,
 	LINE_CTR_FRAMES_DEMUXED_E1O,
+	LINE_CTR_USB_ISO_TRUNC,
 };
 
 enum e1d_line_stat_item {
@@ -77,6 +78,11 @@ enum e1_ts_mode {
 };
 extern const struct value_string e1_ts_mode_names[];
 
+enum e1_framing_mode {
+	E1_FRAMING_MODE_CRC4 = 0,
+	E1_FRAMING_MODE_NO_CRC4,
+};
+
 struct e1_ts {
 	struct e1_line *line;
 	uint8_t id;
@@ -147,6 +153,10 @@ struct e1_line {
 		uint8_t prev_errmask;
 		/*! timer to re-set the rx_crc4_err and rx_alarm above */
 		struct osmo_timer_list timer;
+		/*! current transmitting frame with Sa bits */
+		uint8_t tx_frame;
+		/*! last received frame with Sa bits */
+		uint8_t rx_frame;
 	} ts0;
 
 	/* watchdog timer to catch situations where no more USB data is received */
@@ -155,6 +165,14 @@ struct e1_line {
 		uint32_t rx_bytes;
 	} watchdog;
 
+	/* driver specific data */
+	struct {
+		struct {
+			enum e1_framing_mode tx;
+			enum e1_framing_mode rx;
+		} framing;
+	} usb;
+
 	void *e1gen_priv;
 };
 
@@ -192,9 +210,12 @@ struct e1_intf {
 	struct llist_head lines;
 };
 
+struct osmo_e1dp_server;
+
 struct e1_daemon {
 	void *ctx;
 	struct llist_head interfaces;
+	struct osmo_e1dp_server *srv;
 };
 
 extern const struct octoi_ops e1d_octoi_ops;
